const path = require('path');
const fs = require('fs');
const exec = require('child_process');


const htmlTemp = `<!DOCTYPE html>
<html lang="en">

<head>
    <title></title>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <link rel="stylesheet" href="http://api.alink.aliyun.com/sdk/v1/component.css">
    <link href="./style/main.css" rel="stylesheet">
</head>

<body>
    <!--开心的添加代码吧-->


    

    <script  src="https://api.alink.aliyun.com/sdk/v2/sdk.js"></script>
    <script src="https://api.alink.aliyun.com/sdk/v1/component.js"></script>
</body>
</html>
`;


const jsTemp = `require(['./main.js'],function(main){
    //code write here
    main.init();
});
`;

/**
 * 添加带有工作步骤类型的modelname ，洗衣机，烤箱，电饭煲，豆浆机，面包机，微波炉，压力锅
 */
const CommonType = ['WASHER', 'OVEN', 'RICECOOKER', 'SOYMILKMAKER', 'BREADMACHINE', 'MICROWAVEOVEN', 'PRESSURECOOKER'];

const mainTemp = `define(function(){
    var APP = {
        init:function(){
            this.getDeviceStatus();
            this.bindPushData();
            this.bindEvent();
        },
        getDeviceStatus:function(){

        },
        bindPushData:function(){

        },
        bindEvent:function(){
            
        }
    }
    return APP;
});
`;

const cssTemp = `
    
`;



var argv = require("minimist")(process.argv.slice(2), {
    alias: {
        'silent': 's',
        'port': 'p',
        'hostname': 'h',
        'dir': 'd',
        'log': 'l',
        'fallback': 'f'
    },
    string: ['port', 'hostname', 'fallback'],
    boolean: ['silent', 'log'],
    'default': {
        'port': 8000,
        'dir': process.cwd()
    }
});

let BASEPATH = argv.dir;

let modelName;

exports.CreateAli = (model) => {
    console.log("==============================创建阿里项目模板=========================");
    modelName = model;
    createModelDir();
}

function createModelDir() {
    fs.readdir(BASEPATH, (err, files) => {
        // console.log(files);
        // console.log(files.indexOf(modelName));
        if (!files.includes(modelName)) {
            if (checkModelType()) {
                fs.mkdirSync(`${BASEPATH}/${modelName}`);
                if (global.platform === 'win32') {
                    fs.mkdirSync(`${BASEPATH}/${modelName}/work`);
                    fs.mkdirSync(`${BASEPATH}/${modelName}/stepCommon`);
                    copyFile([{
                            from: path.join(__dirname, 'stepCommon.zip'),
                            to: path.join(BASEPATH + `/${modelName}/stepCommon.zip`),
                            type: 'stepCommon'
                        },
                        {
                            from: path.join(__dirname, 'work.zip'),
                            to: path.join(BASEPATH + `/${modelName}/work.zip`),
                            type: 'work'
                        }
                    ]);
                } else {
                    copyFile([{
                            from: path.join(__dirname, 'stepCommonforMac.zip'),
                            to: path.join(BASEPATH + `/${modelName}/stepCommonforMac.zip`),
                            type: 'stepCommon'
                        },
                        {
                            from: path.join(__dirname, 'workforMac.zip'),
                            to: path.join(BASEPATH + `/${modelName}/workforMac.zip`),
                            type: 'work'
                        }
                    ]);
                }
                creatFiles();
            } else {
                fs.mkdirSync(`${BASEPATH}/${modelName}`);
                creatFiles();
            }
        } else {
            console.log(modelName + ': 已存在');
        }
    });
}


/**
 * 复制指定类型文件所需文件
 * 
 */
function copyFile(arr) {
    //const filePath = from;
    let unzips = [];
    for (let i in arr) {
        const readSM = fs.createReadStream(arr[i].from);
        const writeSM = fs.createWriteStream(arr[i].to);
        readSM.pipe(writeSM);
        setTimeout(() => {
            execCmd(arr[i].to, arr[i].type);
        }, 1000);

    }

}

/**
 * 检查MODEL类型
 * 
 * @returns 
 */
function checkModelType() {
    let modelArry = modelName.split('_');
    for (let i in modelArry) {
        if (CommonType.includes(modelArry[i])) {
            return true;
        }
    }
    return false;
}

function creatFiles() {
    createHtml();
    createJS();
    createStyle();
}

function createHtml() {
    fs.writeFile(`${BASEPATH}/${modelName}/app.html`, htmlTemp, 'utf8', (err) => {
        if (err) {
            throw err;
        }
        console.log('>>创建app.html');
    });

}

function createJS() {
    fs.writeFile(`${BASEPATH}/${modelName}/app.js`, jsTemp, 'utf8', (err) => {
        if (err) {
            throw err;
        }
        console.log('>>创建app.js');
    });
    fs.writeFile(`${BASEPATH}/${modelName}/main.js`, mainTemp, 'utf8', (err) => {
        if (err) {
            throw err;
        }
        console.log('>>创建main.js');
    });

}

function createStyle() {
    fs.mkdir(`${BASEPATH}/${modelName}/style`, () => {
        createStyleFile();
    });

}

function createStyleFile() {
    fs.writeFile(`${BASEPATH}/${modelName}/style/main.css`, cssTemp, 'utf8', (err) => {
        if (err) {
            throw err;
        }
        console.log('>>创建main.css');
    });

}


/**
 * 解压
 * 
 */
// function unzipFn(){
//     //fs.createReadStream(path.resolve(`./package/${modelName}/stepCommon.zip`)).pipe(unzip.Extract(`./package/${modelName}/stepCommon`));
//     var extract = unzip.Extract({path:path.resolve(`./package/${modelName}/stepCommon`)});
//     extract.on('error',(err)=>{
//         console.log('解压错误');
//     });
//     extract.on('finish',()=>{
//         console.log('解压完成');
//     })
//     console.log(path.resolve(`./package/${modelName}/stepCommon.zip`));
//     fs.createReadStream(path.resolve(`./package/${modelName}/stepCommon.zip`)).pipe(extract);
// }


/**
 * 
 * linux解压
 */
function execCmd(arry, type) {
    if (global.platform === 'win32') {
        console.log(`expand -F:* ${arry} ${path.join(`${BASEPATH}/${modelName}/${type}`)}`);
        exec.execSync(`expand -F:* ${arry} ${path.join(`${BASEPATH}/${modelName}/${type}`)}`, { cwd: BASEPATH + `/${modelName}` });
        console.log(`解压${arry}，完成`);
        exec.execSync(`del ${arry}`);
        console.log(`删除${arry}，完成`);
    } else {
        exec.execSync(`unzip ${arry}`, { cwd: BASEPATH + `/${modelName}` });
        console.log(`解压${arry}，完成`);
        exec.execSync(`rm ${arry}`);
        console.log(`删除${arry}，完成`);
    }

}